@@ -76,9 +76,18 @@
 #include <string.h>
 #include <stdio.h>
 
-#include "ferretmacros.h"
+#include "fmtprotos.h"
 #include "ez_delimited_read.h"
+#include "ferret.h"
+#include "FerMem.h"
 
+static int decode_file(char* fname, char *recptr, char *delims, int *skip, int* maxrec, int* reclen, 
+                       int* nfields, int field_type[], int* nrec, DFTYPE** numeric_fields, 
+                       char*** text_fields, DFTYPE bad_flags[], int* status);
+static void decodeRec(char *recptr, char *delims, int* nfields, int field_type[], int rec, 
+                     DFTYPE** numeric_fields, char*** text_fields, DFTYPE bad_flags[], int* status);
+static void analRec(char *recptr, char *delims, int* nfields, int field_type[], int max_fields);
+static char *nexstrtok(char *s1, char *s2);
 
 /*
  *
@@ -100,13 +109,9 @@
  *
  */
 
-
-void FORTRAN(decode_file_jacket)
-		( char* fname, char *recptr, char *delims, int *skip,
-		  int* maxrec, int* reclen, int* nfields,
-		  int field_type[], int* nrec,
-		  int mrlist[], long* mr_ptrs_val,
-		  DFTYPE mr_bad_flags[], char ***mr_c_ptr, int* status)
+void FORTRAN(decode_file_jacket)(char* fname, char *recptr, char *delims, int *skip, int* maxrec, int* reclen, 
+                                 int* nfields, int field_type[], int* nrec, int mrlist[], DFTYPE ***mr_ptrs_ptr, 
+                                 DFTYPE mr_bad_flags[], char ***mr_c_ptr, int* status)
 /* 1/17 --- pre-dynamic memory call
 void FORTRAN(decode_file_jacket)
 		( char* fname, char *recptr, char *delims, int *skip,
@@ -116,11 +121,11 @@ void FORTRAN(decode_file_jacket)
 		  DFTYPE mr_bad_flags[], char ***mr_c_ptr, int* status)
 */
 {
-  DFTYPE** mr_ptrs = (DFTYPE**) *mr_ptrs_val;
-  DFTYPE **numeric_fields  = (DFTYPE **) malloc(sizeof(DFTYPE*) * (*nfields));
-  DFTYPE *bad_flags        = (DFTYPE *)  malloc(sizeof(DFTYPE) * (*nfields));
+  DFTYPE** mr_ptrs = *mr_ptrs_ptr;
+  DFTYPE **numeric_fields  = (DFTYPE **) FerMem_Malloc(sizeof(DFTYPE*) * (*nfields), __FILE__, __LINE__);
+  DFTYPE *bad_flags        = (DFTYPE *)  FerMem_Malloc(sizeof(DFTYPE) * (*nfields), __FILE__, __LINE__);
 
-  char ***text_fields     = (char ***) malloc(sizeof(char**) * (*nfields));
+  char ***text_fields     = (char ***) FerMem_Malloc(sizeof(char**) * (*nfields), __FILE__, __LINE__);
   int i, mr;
   int pinc = 8/sizeof(char*);  /* pointers spaced 8 bytes apart */
 
@@ -157,14 +162,13 @@ void FORTRAN(decode_file_jacket)
   /*
     at last we actually read the file
   */
-  decode_file (fname, recptr, delims, skip, 
-	       maxrec, reclen, nfields,
-	       field_type, nrec,
-	       numeric_fields, text_fields, bad_flags, status);
+  decode_file(fname, recptr, delims, skip, maxrec, reclen, 
+	      nfields, field_type, nrec, numeric_fields, 
+              text_fields, bad_flags, status);
 
-  free(numeric_fields);
-  free(text_fields);
-  free(bad_flags);
+  FerMem_Free(numeric_fields, __FILE__, __LINE__);
+  FerMem_Free(text_fields, __FILE__, __LINE__);
+  FerMem_Free(bad_flags, __FILE__, __LINE__);
 
   return;
 }
@@ -193,11 +197,9 @@ void FORTRAN(decode_file_jacket)
  */
 
 
-int decode_file (char* fname, char *recptr, char *delims, int *skip, 
-			  int* maxrec, int* reclen, int* nfields,
-			  int field_type[], int* nrec, DFTYPE** numeric_fields,
-			  char*** text_fields, DFTYPE bad_flags[], int* status)
-
+static int decode_file(char* fname, char *recptr, char *delims, int *skip, int* maxrec, int* reclen, 
+                       int* nfields, int field_type[], int* nrec, DFTYPE** numeric_fields, 
+                       char*** text_fields, DFTYPE bad_flags[], int* status)
 {
 
   FILE *fp;
@@ -366,10 +368,8 @@ int FORTRAN(anal_file) (char* fname, char *recptr, char *delims, int* skip,
  *
  */
 
-int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
-	      int rec,
-	      DFTYPE** numeric_fields, char*** text_fields, DFTYPE bad_flags[],
-	      int* status)
+static void decodeRec(char *recptr, char *delims, int* nfields, int field_type[], int rec, 
+                     DFTYPE** numeric_fields, char*** text_fields, DFTYPE bad_flags[], int* status)
 {
 
   char *p, *pnext, str1[2], errstr[2];
@@ -402,7 +402,7 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
     } else if (p==NULL || *p == '\0') {
       /* missing data field */
       if ( field_type[i] == FTYP_CHARACTER ) {
-	(*(text_fields+i))[rec*pinc] = (char *) malloc(sizeof(char)*2);
+	(*(text_fields+i))[rec*pinc] = (char *) FerMem_Malloc(sizeof(char)*2, __FILE__, __LINE__);
 	strcpy( (*(text_fields+i))[rec*pinc], blankstr );
       }
       else {
@@ -460,7 +460,7 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 	  /* check for yyyy/mm/dd */
 	  if (idummy1 > 1800) {
 		  (*(numeric_fields+i))[rec] =
-		  days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+		  FORTRAN(days_from_day0)(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
 		  (*(numeric_fields+i))[rec] = rdum;
 		  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
 		  break;
@@ -475,14 +475,14 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 	  }
 
 	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy3,&idummy1,&idummy2,&rdum,status);
+	    FORTRAN(days_from_day0)(&days_1900,&idummy3,&idummy1,&idummy2,&rdum,status);
 	  (*(numeric_fields+i))[rec] = rdum;
 	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
 	/* force dates with dashes "-" to be in yyyy-mm-dd format *kob* */
 	} else if (sscanf(p,"%4d-%2d-%2d%1s",
 			  &idummy1,&idummy2,&idummy3,errstr) == 3) {
 	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+	    FORTRAN(days_from_day0)(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
 	  (*(numeric_fields+i))[rec] = rdum; 
 	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
 	  }
@@ -492,7 +492,7 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 	      && idummy2>=1 && idummy2<=12
 	      && idummy3>=1 && idummy3<=31 ) {
 	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+	    FORTRAN(days_from_day0)(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
 	  (*(numeric_fields+i))[rec] = rdum; }
 	else
 	  (*(numeric_fields+i))[rec] = bad_flags[i];
@@ -522,7 +522,7 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 	  /* check for yyyy/mm/dd */
 	  if (idummy1 > 1800) {
 		  (*(numeric_fields+i))[rec] =
-		  days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+		  FORTRAN(days_from_day0)(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
 		  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
 	  } else {
 
@@ -535,7 +535,7 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 		  }
 		  
 		  (*(numeric_fields+i))[rec] =
-		  days_from_day0_(&days_1900,&idummy3,&idummy1,&idummy2,&rdum,status);
+		  FORTRAN(days_from_day0)(&days_1900,&idummy3,&idummy1,&idummy2,&rdum,status);
 		  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
 	  }
 
@@ -552,26 +552,20 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 		if (ndum == 5) tpart = idummy4 + idummy5/60.;
 
 	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+	    FORTRAN(days_from_day0)(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
 	  (*(numeric_fields+i))[rec] = rdum + tpart/24.; 
 	  if (tpart == -999) (*(numeric_fields+i))[rec] = bad_flags[i];
 	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
 	  }
 
 	/* check for yyyymmdd date */
-	else if ( (sscanf(p,"%4d%2d%2d%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,str1) >= 4)
+	else if ( (sscanf(p,"%4d%2d%2d%1s",&idummy1,&idummy2,&idummy3,str1) == 3)
 	      && idummy1>0
 	      && idummy3>=1 && idummy3<=12
 	      && idummy2>=1 && idummy2<=31 ) {
-		ndum = sscanf(p,"%4d%2d%2d%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,str1);
-		tpart = -999;	
-		if (ndum == 6) tpart = idummy4 + idummy5/60. + dummy/3600.;
-		if (ndum == 5) tpart = idummy4 + idummy5/60.;	
-
 		(*(numeric_fields+i))[rec] =
-		days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
-		(*(numeric_fields+i))[rec] = rdum + tpart/24.; 	
-	    if (tpart == -999) (*(numeric_fields+i))[rec] = bad_flags[i];
+		FORTRAN(days_from_day0)(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+		(*(numeric_fields+i))[rec] = rdum;
 		if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
 	  }
 	else
@@ -593,7 +587,7 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 	  /* check for yyyy/mm/dd */
 	  if (idummy1 > 1800) {
 		  (*(numeric_fields+i))[rec] =
-		  days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+		  FORTRAN(days_from_day0)(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
 		  (*(numeric_fields+i))[rec] = rdum;
 		  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
 		  break;
@@ -608,14 +602,14 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 	  }
 
 	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy3,&idummy2,&idummy1,&rdum,status);
+	    FORTRAN(days_from_day0)(&days_1900,&idummy3,&idummy2,&idummy1,&rdum,status);
 	  (*(numeric_fields+i))[rec] = rdum;
 	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
 	  /* force dates with dashes "-" to be in yyyy-mm-dd format *kob* */
 	} else if (sscanf(p,"%4d-%2d-%2d%1s",
 			  &idummy1,&idummy2,&idummy3,errstr) == 3) {
 	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+	    FORTRAN(days_from_day0)(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
 	  (*(numeric_fields+i))[rec] = rdum;
 	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i]; }
 	/* add check for yyyyddmm euro date *kob* */
@@ -624,7 +618,7 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 	      && idummy3>=1 && idummy3<=12
 	      && idummy2>=1 && idummy2<=31 ) {
 	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy1,&idummy3,&idummy2,&rdum,status);
+	    FORTRAN(days_from_day0)(&days_1900,&idummy1,&idummy3,&idummy2,&rdum,status);
 	  (*(numeric_fields+i))[rec] = rdum;
 	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i]; }
 	else
@@ -655,7 +649,7 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 	  /* check for yyyy/mm/dd */
 	  if (idummy1 > 1800) {
 		  (*(numeric_fields+i))[rec] =
-		  days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+		  FORTRAN(days_from_day0)(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
 
 	  } else {
 
@@ -668,7 +662,7 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 		  }
 		  
 		  (*(numeric_fields+i))[rec] =
-		  days_from_day0_(&days_1900,&idummy3,&idummy2,&idummy1,&rdum,status);
+		  FORTRAN(days_from_day0)(&days_1900,&idummy3,&idummy2,&idummy1,&rdum,status);
 	  }
 
 	  (*(numeric_fields+i))[rec] = rdum + tpart/24.;
@@ -685,26 +679,20 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 		if (ndum == 5) tpart = idummy4 + idummy5/60.;
 
 	  (*(numeric_fields+i))[rec] =
-	    days_from_day0_(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
+	    FORTRAN(days_from_day0)(&days_1900,&idummy1,&idummy2,&idummy3,&rdum,status);
 	  (*(numeric_fields+i))[rec] = rdum + tpart/24.; 
 	  if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];
 	  if (tpart == -999) (*(numeric_fields+i))[rec] = bad_flags[i];
 	  }
 
 	/* add check for yyyyddmm euro date *kob* */
-	else if ( (sscanf(p,"%4d%2d%2d%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,str1) >= 4)
+	else if ( (sscanf(p,"%4d%2d%2d%1s",&idummy1,&idummy2,&idummy3,str1) == 3)
 	      && idummy1>0
 	      && idummy3>=1 && idummy3<=12
 	      && idummy2>=1 && idummy2<=31 ) {
-		ndum = sscanf(p,"%4d%2d%2d%1s",&idummy1,&idummy2,&idummy3,&idummy4,&idummy5,&dummy,str1);
-		tpart = -999;	
-		if (ndum == 6) tpart = idummy4 + idummy5/60. + dummy/3600.;
-		if (ndum == 5) tpart = idummy4 + idummy5/60.;	
-
 		(*(numeric_fields+i))[rec] =
-		days_from_day0_(&days_1900,&idummy1,&idummy3,&idummy2,&rdum,status);
-		(*(numeric_fields+i))[rec] = rdum + tpart/24.;
-	    if (tpart == -999) (*(numeric_fields+i))[rec] = bad_flags[i];
+		FORTRAN(days_from_day0)(&days_1900,&idummy1,&idummy3,&idummy2,&rdum,status);
+		(*(numeric_fields+i))[rec] = rdum;
 	    if (*status != 3) (*(numeric_fields+i))[rec] = bad_flags[i];  }
 	else
 	  (*(numeric_fields+i))[rec] = bad_flags[i];
@@ -745,7 +733,7 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
 	    p++;
 	  }
 	  (*(text_fields+i))[rec*pinc] =
-	    (char *) malloc(sizeof(char)*(strlen(p)+1));
+	    (char *) FerMem_Malloc(sizeof(char)*(strlen(p)+1), __FILE__, __LINE__);
 	  strcpy( (*(text_fields+i))[rec*pinc], p );
 	}
 	break;
@@ -781,14 +769,13 @@ int decodeRec(char *recptr, char *delims, int* nfields, int field_type[],
  *
  */
 
-void analRec(char *recptr, char *delims, int* nfields, int field_type[],
-	    int max_fields)
+static void analRec(char *recptr, char *delims, int* nfields, int field_type[], int max_fields)
 {
 
   char *p, *pnext, pstart[256], str1[2], latlon1[2];
   double dummy;
 
-  int idummy1, idummy2, idummy3, i, nfields_in;
+  int idummy1, idummy2, idummy3, nfields_in;
 
   p = recptr;
   nfields_in = *nfields;
@@ -925,8 +912,7 @@ void analRec(char *recptr, char *delims, int* nfields, int field_type[],
  *
  */
 
-char *nexstrtok(char *s1, char *s2)
-
+static char *nexstrtok(char *s1, char *s2)
      /*
        like strtok but sensitive to multiple (non-white space) delimiters
        as significant. For example, 2 commas together indicate a missing field.
@@ -975,11 +961,13 @@ char *nexstrtok(char *s1, char *s2)
 void FORTRAN(save_delimited_info) (int *nfields, int field_type[],
 				   char *delim, DelimFileInfo **ptr)
 {
-  DelimFileInfo *fi = (DelimFileInfo *) calloc(1, sizeof(DelimFileInfo));
-  int* _field_type  = (int *) malloc(sizeof(int) * (*nfields));
-  char* _delim      = (char *) malloc(sizeof(char) * (int)strlen(delim));
+  DelimFileInfo *fi = (DelimFileInfo *) FerMem_Malloc(sizeof(DelimFileInfo), __FILE__, __LINE__);
+  int* _field_type  = (int *) FerMem_Malloc(sizeof(int) * (*nfields), __FILE__, __LINE__);
+  char* _delim      = (char *) FerMem_Malloc(sizeof(char) * (int)strlen(delim) + 1, __FILE__, __LINE__);
   int i;
 
+  memset(fi, 0, sizeof(DelimFileInfo));
+
   for (i=0; i<*nfields; i++)
     _field_type[i] = field_type[i];
 
@@ -997,7 +985,7 @@ void FORTRAN(get_delimited_info) (int *nfields, int field_type[],
 				   char *delim, DelimFileInfo **ptr)
 {
 
-  int i,iout;
+  int i;
   DelimFileInfo *fi = *ptr;
 
   *nfields = fi->nfields;
@@ -1010,9 +998,15 @@ void FORTRAN(get_delimited_info) (int *nfields, int field_type[],
 void FORTRAN(delete_delimited_info) (DelimFileInfo **ptr)
 {
   DelimFileInfo *fi = *ptr;
-  free(fi->fieldType);
-  free(fi->delim);
-  free(fi);
+  if ( fi != NULL ) {
+    if ( fi->fieldType != NULL )
+      FerMem_Free(fi->fieldType, __FILE__, __LINE__);
+    if ( fi->delim != NULL )
+      FerMem_Free(fi->delim, __FILE__, __LINE__);
+    memset(fi, 0, sizeof(DelimFileInfo));
+    FerMem_Free(fi, __FILE__, __LINE__);
+    *ptr = NULL;
+  }
   return;
 }
 
