@@ -41,15 +41,12 @@
 
 #include <Python.h> /* make sure Python.h is first */
 #include <stdio.h>
+#include <stdlib.h>
+#include "fmtprotos.h"
 #include "string_array.h"
 
-void string_array_find_caseblind_(  double *string_array_header,
-			            char *test_string,
-                                    int *test_len,
-                                    int *result_array,
-                                    int *result_array_size,
-                                    int *num_indices
-                                  )
+void FORTRAN(string_array_find_caseblind)(void **string_array_header, char *test_string, int *test_len, 
+                                          int *result_array, int *result_array_size, int *num_indices)
 {
    int i,j=0;
    int true_test_len, true_model_len, array_size, 
@@ -57,15 +54,14 @@ void string_array_find_caseblind_(  double *string_array_header,
    SA_Head * head;
    List_Node *bucket, *p;
    char * model_string;
-   int match=0, is_quoted=0;
+   int match;
 
- 
-   if(*(SA_Head**)string_array_header){
-      head = *((SA_Head**)string_array_header);
+   head = *string_array_header;
+   if( head != NULL ){
       array_size = head->array_size;
       string_size = head->string_size;
  
-      tm_get_strlen_(&true_test_len, test_len, test_string);
+      FORTRAN(tm_get_strlen)(&true_test_len, test_len, test_string);
  
  
       hash_value = string_array_hash(test_string, true_test_len, 0, array_size);
@@ -82,7 +78,7 @@ void string_array_find_caseblind_(  double *string_array_header,
 
       for(p=bucket; p; p=p->next) {
 	  model_string=&(head->string_array[(p->index-1)*string_size]);
-          string_array_get_strlen_(string_array_header, &(p->index), &true_model_len);
+          FORTRAN(string_array_get_strlen)(string_array_header, &(p->index), &true_model_len);
 
           match = 0;
 
@@ -109,7 +105,7 @@ void string_array_find_caseblind_(  double *string_array_header,
       }
    }
    else{
-       printf("\nString array not initialized yet!");
+       printf("\nString array not initialized yet (string_array_find_caseblind)!\n");
    }
    *num_indices = j;
 }
